@@ -20,10 +20,12 @@ along with this program. If not, see <http://www.gnu.org/licenses/>.
 #include "Python.h"
 
 
+#define WE_GOT_A_PROBLEM -2
+
 #define MIN3(a, b, c) ((a) < (b) ? ((a) < (c) ? (a) : (c)) : ((b) < (c) ? (b) : (c)))
 
 PyDoc_STRVAR(hamming_doc,
-"hamming(seq1, seq2, normalized=False) -> hamming distance\n\
+"hamming(seq1, seq2, normalized=False)\n\
 \n\
 Compute the Hamming distance between the two sequences `seq1` and `seq2`.\n\
 The Hamming distance is the number of differing items in two ordered sequences\n\
@@ -35,11 +37,12 @@ If `normalized` evaluates to `False`, the return value will be an integer betwee
 will be a float between 0 and 1 included, where 0 means equal, and 1 totally different.\n\
 Normalized hamming distance is computed as:\n\
 \n\
-    hamming_dist / len(seq1)\
+    0.0                         if len(seq1) == 0\n\
+    hamming_dist / len(seq1)    otherwise\
 ");
 
 PyDoc_STRVAR(levenshtein_doc,
-"levenshtein(seq1, seq2, normalized=False) -> levenshtein distance\n\
+"levenshtein(seq1, seq2, normalized=False, max_dist=2)\n\
 \n\
 Compute the Levenshtein distance between the two sequences `seq1` and `seq2`.\n\
 The Levenshtein distance is the minimum number of edit operations necessary for\n\
@@ -51,27 +54,48 @@ transforming one sequence into the other. The edit operations allowed are:\n\
 \n\
 If `normalized` evaluates to `False`, the return value will be an integer between\n\
 0 and the length of the sequences provided, edge values included; otherwise, it\n\
-will be a float between 0 and 1 included, where 0 means equal, and 1 totally different.\n\
-Normalized levenshtein distance is computed as:\n\
+will be a float between 0 and 1 included, where 0 means equal, and 1 totally\n\
+different. Normalized levenshtein distance is computed as:\n\
 \n\
-    lev_dist / max(len(seq1), len(seq2))\
+    0.0                                   if len(seq1) == len(seq2) == 0\n\
+    lev_dist / max(len(seq1), len(seq2))  otherwise\n\
+\n\
+The `max_dist` parameter controls at which moment we should stop computing the\n\
+distance between the provided sequences. If it is a negative int, the distance\n\
+will be computed until the sequences are exhausted; otherwise, the computation\n\
+will stop at the moment the calculated distance is higher than `max_dist`, and\n\
+then return -1. For example:\n\
+\n\
+    >>> levenshtein(\"abc\", \"abcd\", max_dist=1)  # dist=1\n\
+    1\n\
+    >>> levenshtein(\"abc\", \"abcde\", max_dist=1) # dist=2\n\
+    -1\n\
+\n\
+This can be a time saver if you're not interested in absolute distance.\
 ");
 
-PyDoc_STRVAR(quick_levenshtein_doc,
-"quick_levenshtein(str1, str2) -> levenshtein distance\n\
+PyDoc_STRVAR(fast_comp_doc,
+"fast_comp(str1, str2, transpositions=False)\n\
 \n\
-Compute the levenshtein distance between the two strings `str1` and `str2` up\n\
-to a maximum of 2 included, and return it. If the edit distance between the two\n\
-strings is higher than that, -1 is returned.\n\
+Compute the distance between the two strings `str1` and `str2` up to a maximum\n\
+of 2 included, and return it. If the edit distance between the two strings is higher\n\
+than that, -1 is returned.\n\
 \n\
-This can be a lot faster than `levenshtein`.\n\
+`str1` and `str2` are expected to be unicode strings. If `transpositions` is `True`,\n\
+transpositions will be taken into account when computing the distance between the\n\
+submitted strings. This can make a difference, e.g.:\n\
 \n\
-The algorithm comes from `http://writingarchives.sakura.ne.jp/fastcomp`. The python\n\
-code is the original one, and it has been rewritten in C for a large performance gain.\
+    >>> fast_comp(\"abc\", \"bac\", transpositions=False)\n\
+    2\n\
+    >>> fast_comp(\"abc\", \"bac\", transpositions=True)\n\
+    1\n\
+\n\
+The algorithm comes from `http://writingarchives.sakura.ne.jp/fastcomp`. I've added\n\
+transpositions support to the original code, and rewritten it in C.\
 ");
 
-PyDoc_STRVAR(iquick_levenshtein_doc,
-"iquick_levenshtein(str1, strs) -> (distance, str2)..\n\
+PyDoc_STRVAR(ifast_comp_doc,
+"ifast_comp(str1, strs, transpositions=False)\n\
 \n\
 Return an iterator over all the strings in `strs` which distance from `str1`\n\
 is lower or equal to 2. The strings which distance from the reference string\n\
@@ -79,27 +103,29 @@ is higher than that are dropped.\n\
 \n\
     `str1`: the reference unicode string.\n\
     `strs`: an iterable of unicode strings (can be a generator).\n\
+    `transpositions` has the same sense than in `fast_comp`.\n\
 \n\
 The return value is a series of pairs (distance, string).\n\
 \n\
-This is intended to be used to filter from a long list of strings the ones that\n\
-are unlikely to be good spelling suggestions for the reference string (distance 2\n\
-being considered a high enough value in most cases).\n\
-\n\
 This is faster than `levensthein` by an order of magnitude, so use this if you're\n\
 only interested in strings which are below distance 2 from the reference string.\n\
+If you need a different threshold than distance 2, see the `max_dist` parameter\n\
+in `levenshtein`.\n\
 \n\
 You might want to call `sorted()` on the iterator to get the results in a\n\
 significant order:\n\
 \n\
-    >>> g = iquick_levenshtein(\"foo\", [\"fo\", \"bar\", \"foob\", \"foo\", \"foobaz\"])\n\
+    >>> g = ifast_comp(\"foo\", [\"fo\", \"bar\", \"foob\", \"foo\", \"foobaz\"])\n\
     >>> sorted(g)\n\
     [(0, 'foo'), (1, 'fo'), (1, 'foob')]\
 ");
 
 
+
+/* Hamming */
+
 static Py_ssize_t
-uni_hamming(PyObject *str1, PyObject *str2, Py_ssize_t len1, Py_ssize_t len2)
+uni_hamming(PyObject *str1, PyObject *str2, Py_ssize_t len)
 {
 	Py_ssize_t i, dist = 0;
 	
@@ -108,7 +134,7 @@ uni_hamming(PyObject *str1, PyObject *str2, Py_ssize_t len1, Py_ssize_t len2)
 	int kind1 = PyUnicode_KIND(str1);
 	int kind2 = PyUnicode_KIND(str2);
     
-	for (i = 0; i < len1; i++) {
+	for (i = 0; i < len; i++) {
 		if (PyUnicode_READ(kind1, ptr1, i) != PyUnicode_READ(kind2, ptr2, i))
 			dist++;
 	}
@@ -118,11 +144,11 @@ uni_hamming(PyObject *str1, PyObject *str2, Py_ssize_t len1, Py_ssize_t len2)
 
 
 static Py_ssize_t
-rich_hamming(PyObject *seq1, PyObject *seq2, Py_ssize_t len1, Py_ssize_t len2)
+rich_hamming(PyObject *seq1, PyObject *seq2, Py_ssize_t len)
 {
 	Py_ssize_t i, comp, dist = 0;
 	
-	for (i = 0; i < len1; i++) {
+	for (i = 0; i < len; i++) {
 		comp = PyObject_RichCompareBool(
 			PySequence_Fast_GET_ITEM(seq1, i),
 			PySequence_Fast_GET_ITEM(seq2, i),
@@ -131,7 +157,7 @@ rich_hamming(PyObject *seq1, PyObject *seq2, Py_ssize_t len1, Py_ssize_t len2)
 			Py_DECREF(seq1);
 			Py_DECREF(seq2);
 			PyErr_SetString(PyExc_RuntimeError, "failed to compare objects");
-			return -1;
+			return WE_GOT_A_PROBLEM;
 		}
 		if (!comp)
 			dist++;
@@ -168,7 +194,7 @@ hamming(PyObject *self, PyObject *args, PyObject *kwargs)
 			PyErr_SetString(PyExc_ValueError, "expected two objects of the same length");
 			return NULL;
 		}
-		dist = uni_hamming(arg1, arg2, len1, len2);
+		dist = uni_hamming(arg1, arg2, len1);
 	}
 	
 	else if (PySequence_Check(arg1) && PySequence_Check(arg2)) {
@@ -197,7 +223,7 @@ hamming(PyObject *self, PyObject *args, PyObject *kwargs)
 			PyErr_SetString(PyExc_ValueError, "expected two objects of the same len");
 			return NULL;
 		}
-		dist = rich_hamming(seq1, seq2, len1, len2);
+		dist = rich_hamming(seq1, seq2, len1);
 	}
 	
 	else {
@@ -205,7 +231,7 @@ hamming(PyObject *self, PyObject *args, PyObject *kwargs)
 		return NULL;
 	}
 	
-	if (dist == -1)
+	if (dist == WE_GOT_A_PROBLEM)
 		return NULL;
 	
 	if (do_normalize) {
@@ -218,118 +244,133 @@ hamming(PyObject *self, PyObject *args, PyObject *kwargs)
 }
 
 
+
+/* Levenshtein */
+
+
+#define OUT_OF_BOUND -1
+
 static Py_ssize_t
-uni_levenshtein(PyObject *str1, PyObject *str2, Py_ssize_t len1, Py_ssize_t len2)
+minimum(Py_ssize_t *column, Py_ssize_t len)
+{
+	Py_ssize_t i;
+	/* column should not be empty, we already checked this */
+	Py_ssize_t min = column[0];
+	
+	for (i = 1; i < len; i++) {
+		if (column[i] < min)
+			min = column[i];
+	}
+	return min;
+}
+
+
+static Py_ssize_t
+uni_levenshtein(PyObject *str1, PyObject *str2,
+	Py_ssize_t len1, Py_ssize_t len2, Py_ssize_t max_dist)
 {
 	Py_ssize_t x, y, last, old;
 	Py_ssize_t *column;
 	int kind1, kind2;
 	void *ptr1, *ptr2;
 	unsigned short cost;
-	
-	if (len1 == 0)
-		return len2;
-	if (len2 == 0)
-		return len1;
 
-	if (len1 < len2) {
-		PyObject *temp_str = str1;
-		Py_ssize_t temp_len = len1;
-		str1 = str2;
-		len1 = len2;
-		str2 = temp_str;
-		len2 = temp_len;
+	if (max_dist >= 0 && (len1 - len2) > max_dist)
+		return OUT_OF_BOUND;
+	else {
+		if (len1 == 0)
+			return len2;
+		if (len2 == 0)
+			return len1;
 	}
 
 	ptr1 = PyUnicode_DATA(str1);
 	ptr2 = PyUnicode_DATA(str2);
 	kind1 = PyUnicode_KIND(str1);
 	kind2 = PyUnicode_KIND(str2);
-	
-	column = (Py_ssize_t*) malloc((len1 + 1) * sizeof(Py_ssize_t));
+
+	column = (Py_ssize_t*) malloc((len2 + 1) * sizeof(Py_ssize_t));
 	if (column == NULL) {
-		Py_DECREF(str1);
-		Py_DECREF(str2);
 		PyErr_SetString(PyExc_RuntimeError, "no memory");
-		return -1;
+		return WE_GOT_A_PROBLEM;
 	}
 
-	for (y = 1; y <= len1; y++)
+	for (y = 1 ; y <= len2; y++)
 		column[y] = y;
-	for (x = 1; x <= len2; x++) {
+	for (x = 1 ; x <= len1; x++) {
 		column[0] = x;
-		for (y = 1, last = x - 1; y <= len1; y++) {
+		for (y = 1, last = x - 1; y <= len2; y++) {
 			old = column[y];
-			cost = (PyUnicode_READ(kind1, ptr1, y - 1) != PyUnicode_READ(kind2, ptr2, x - 1));
+			cost = (PyUnicode_READ(kind1, ptr1, x - 1) != PyUnicode_READ(kind2, ptr2, y - 1));
 			column[y] = MIN3(column[y] + 1, column[y - 1] + 1, last + cost);
 			last = old;
 		}
+		if (max_dist >= 0 && minimum(column, len2 + 1) > max_dist) {
+			free(column);
+			return OUT_OF_BOUND;
+		}
 	}
 
 	free(column);
-	return column[len1];
+	
+	if (max_dist >= 0 && column[len2] > max_dist)
+		return OUT_OF_BOUND;
+	return column[len2];
 }
 
 
 static Py_ssize_t
-rich_levenshtein(PyObject *seq1, PyObject *seq2, Py_ssize_t len1, Py_ssize_t len2)
+rich_levenshtein(PyObject *seq1, PyObject *seq2,
+	Py_ssize_t len1, Py_ssize_t len2, Py_ssize_t max_dist)
 {
 	Py_ssize_t *column;
 	Py_ssize_t x, y, last, old;
 	int comp;
-	
-	if (len1 == 0 || len2 == 0) {
-		Py_DECREF(seq1);
-		Py_DECREF(seq2);
+
+	if (max_dist >= 0 && (len1 - len2) > max_dist)
+		return OUT_OF_BOUND;
+	else {
 		if (len1 == 0)
 			return len2;
-		return len1;
+		if (len2 == 0)
+			return len1;
 	}
 	
-	if (len1 < len2) {
-		Py_ssize_t len1b = len1;
-		PyObject *seq1b = seq1;
-		len1 = len2;
-		seq1 = seq2;
-		len2 = len1b;
-		seq2 = seq1b;
-	}
-		
-	column = (Py_ssize_t*) malloc((len1 + 1) * sizeof(Py_ssize_t));
+	column = (Py_ssize_t*) malloc((len2 + 1) * sizeof(Py_ssize_t));
 	if (column == NULL) {
-		Py_DECREF(seq1);
-		Py_DECREF(seq2);
 		PyErr_SetString(PyExc_RuntimeError, "no memory");
-		return -1;
+		return WE_GOT_A_PROBLEM;
 	}
 
-	for (y = 1; y <= len1; y++)
+	for (y = 1; y <= len2; y++)
 		column[y] = y;
-	for (x = 1; x <= len2; x++) {
+	for (x = 1; x <= len1; x++) {
 		column[0] = x;
-		for (y = 1, last = x - 1; y <= len1; y++) {
+		for (y = 1, last = x - 1; y <= len2; y++) {
 			old = column[y];
 			comp = PyObject_RichCompareBool(
-				PySequence_Fast_GET_ITEM(seq1, y - 1),
-				PySequence_Fast_GET_ITEM(seq2, x - 1),
+				PySequence_Fast_GET_ITEM(seq1, x - 1),
+				PySequence_Fast_GET_ITEM(seq2, y - 1),
 				Py_EQ);
 			if (comp == -1) {
-				Py_DECREF(seq1);
-				Py_DECREF(seq2);
 				free(column);
 				PyErr_SetString(PyExc_RuntimeError, "failed to compare objects");
-				return -1;
+				return WE_GOT_A_PROBLEM;
 			}
 			column[y] = MIN3(column[y] + 1, column[y - 1] + 1, last + (comp != 1));
 			last = old;
 		}
+		if (max_dist >= 0 && minimum(column, len2 + 1) > max_dist) {
+			free(column);
+			return OUT_OF_BOUND;
+		}
 	}
-
-	Py_DECREF(seq1);
-	Py_DECREF(seq2);
+	
 	free(column);
 	
-	return column[len1];
+	if (max_dist >= 0 && column[len2] > max_dist)
+		return OUT_OF_BOUND;
+	return column[len2];
 }
 
 
@@ -337,12 +378,13 @@ static PyObject *
 levenshtein(PyObject *self, PyObject *args, PyObject *kwargs)
 {
 	PyObject *arg1, *arg2;
-	Py_ssize_t len1, len2, dist;
+	Py_ssize_t len1, len2, dist=-1;
 	int do_normalize = 0;
-	static char *keywords[] = {"arg1", "arg2", "normalized", NULL};
+	Py_ssize_t max_dist = -1;
+	static char *keywords[] = {"arg1", "arg2", "normalized", "max_dist", NULL};
 	
 	if (!PyArg_ParseTupleAndKeywords(args, kwargs,
-		"OO|p:levenshtein", keywords, &arg1, &arg2, &do_normalize))
+		"OO|pn:levenshtein", keywords, &arg1, &arg2, &do_normalize, &max_dist))
 		return NULL;
 
 	if (PyUnicode_Check(arg1) && PyUnicode_Check(arg2)) {
@@ -351,10 +393,15 @@ levenshtein(PyObject *self, PyObject *args, PyObject *kwargs)
 			return NULL;
 		if (PyUnicode_READY(arg2) != 0)
 			return NULL;
+		
 		len1 = PyUnicode_GET_LENGTH(arg1);
 		len2 = PyUnicode_GET_LENGTH(arg2);
-
-		dist = uni_levenshtein(arg1, arg2, len1, len2);
+		
+		if (len1 > len2)
+			dist = uni_levenshtein(arg1, arg2, len1, len2, max_dist);
+		else
+			dist = uni_levenshtein(arg2, arg1, len2, len1, max_dist);
+	
 	}
 
 	else if (PySequence_Check(arg1) && PySequence_Check(arg2)) {
@@ -378,7 +425,13 @@ levenshtein(PyObject *self, PyObject *args, PyObject *kwargs)
 			return NULL;
 		}
 	
-		dist = rich_levenshtein(arg1, arg2, len1, len2);
+		if (len1 > len2)
+			dist = rich_levenshtein(arg1, arg2, len1, len2, max_dist);
+		else
+			dist = rich_levenshtein(arg2, arg1, len2, len1, max_dist);
+		
+		Py_DECREF(seq1);
+		Py_DECREF(seq2);
 	}
 	
 	else {
@@ -386,7 +439,7 @@ levenshtein(PyObject *self, PyObject *args, PyObject *kwargs)
 		return NULL;
 	}
 
-	if (dist == -1)
+	if (dist == WE_GOT_A_PROBLEM)
 		return NULL;
 	
 	if (do_normalize) {
@@ -400,11 +453,12 @@ levenshtein(PyObject *self, PyObject *args, PyObject *kwargs)
 }
 
 
-/* Quick levenshtein */
 
+/* Fast comp and co. */
 
 static short
-uni_quick_levenshtein(void *ptr1, void *ptr2, int kind1, int kind2, Py_ssize_t len1, Py_ssize_t len2)
+uni_fast_comp(void *ptr1, void *ptr2, int kind1, int kind2,
+	Py_ssize_t len1, Py_ssize_t len2, int transpositions)
 {
 	char *models[3];
 	short m, cnt, res = 3;
@@ -444,13 +498,21 @@ uni_quick_levenshtein(void *ptr1, void *ptr2, int kind1, int kind2, Py_ssize_t l
 				if (c > 2)
 					break;
 				
-				if (models[m][c - 1] == 'd')
-					i++;
-				else if (models[m][c - 1] == 'i')
-					j++;
+				if (transpositions && ldiff != 2 && i < len1 - 1 && j < len2 - 1 && \
+					PyUnicode_READ(kind1, ptr1, i + 1) == PyUnicode_READ(kind2, ptr2, j) && \
+					PyUnicode_READ(kind1, ptr1, i) == PyUnicode_READ(kind2, ptr2, j + 1)) {
+					i = i + 2;
+					j = j + 2;
+				}
 				else {
-					i++;
-					j++;
+					if (models[m][c - 1] == 'd')
+						i++;
+					else if (models[m][c - 1] == 'i')
+						j++;
+					else {
+						i++;
+						j++;
+					}
 				}
 			}
 			else {
@@ -490,14 +552,20 @@ uni_quick_levenshtein(void *ptr1, void *ptr2, int kind1, int kind2, Py_ssize_t l
 	return res;
 }
 
+
 static PyObject *
-quick_levenshtein(PyObject *self, PyObject *args)
+fast_comp(PyObject *self, PyObject *args, PyObject *kwargs)
 {
 	PyObject *arg1, *arg2;
 	Py_ssize_t len1, len2;
+	void *ptr1, *ptr2;
+	int kind1, kind2;
 	short dist;
+	int transpositions = 0;
+	static char *keywords[] = {"arg1", "arg2", "transpositions", NULL};
 
-	if (!PyArg_ParseTuple(args, "UU:quick_levenshtein", &arg1, &arg2))
+	if (!PyArg_ParseTupleAndKeywords(args, kwargs, "UU|p:fast_comp",
+		keywords, &arg1, &arg2, &transpositions))
 		return NULL;
 
 	if (PyUnicode_READY(arg1) != 0)
@@ -507,43 +575,47 @@ quick_levenshtein(PyObject *self, PyObject *args)
 
 	len1 = PyUnicode_GET_LENGTH(arg1);
 	len2 = PyUnicode_GET_LENGTH(arg2);
-	void *ptr1 = PyUnicode_DATA(arg1);
-	void *ptr2 = PyUnicode_DATA(arg2);
-	int kind1 = PyUnicode_KIND(arg1);
-	int kind2 = PyUnicode_KIND(arg2);
+	ptr1 = PyUnicode_DATA(arg1);
+	ptr2 = PyUnicode_DATA(arg2);
+	kind1 = PyUnicode_KIND(arg1);
+	kind2 = PyUnicode_KIND(arg2);
 	
 	if (len1 > len2)
-		dist = uni_quick_levenshtein(ptr1, ptr2, kind1, kind2, len1, len2);
+		dist = uni_fast_comp(ptr1, ptr2, kind1, kind2, len1, len2, transpositions);
 	else
-		dist = uni_quick_levenshtein(ptr2, ptr1, kind2, kind1, len2, len1);
+		dist = uni_fast_comp(ptr2, ptr1, kind2, kind1, len2, len1, transpositions);
 	
 	return Py_BuildValue("h", dist);	
 }
 
 
 typedef struct {
-    PyObject_HEAD
-    PyObject *itor;
-    PyObject *str1;
-    void *ptr1;
-    int kind1;
-    Py_ssize_t len1;
+	PyObject_HEAD
+	PyObject *itor;
+	PyObject *str1;
+	void *ptr1;
+	int kind1;
+	Py_ssize_t len1;
+	int transpos;
 } ItorState;
 
 
 static void itor_dealloc(ItorState *state)
 {
-	Py_DECREF(state->str1);
+	Py_XDECREF(state->str1);
 	Py_XDECREF(state->itor);
 	Py_TYPE(state)->tp_free(state);
 }
 
 
-static PyObject * iquick_levenshtein(PyTypeObject *type, PyObject *args, PyObject *kwargs)
+static PyObject * ifast_comp(PyTypeObject *type, PyObject *args, PyObject *kwargs)
 {
 	PyObject *arg1, *arg2, *itor;
+	int transpositions = 0;
+	static char *keywords[] = {"str1", "strs", "transpositions", NULL};
 
-	if (!PyArg_ParseTuple(args, "UO:iquick_levenshtein", &arg1, &arg2))
+	if (!PyArg_ParseTupleAndKeywords(args, kwargs, "UO|p:ifast_comp",
+		keywords, &arg1, &arg2, &transpositions))
 		return NULL;
 
 	if (PyUnicode_READY(arg1) != 0)
@@ -565,12 +637,13 @@ static PyObject * iquick_levenshtein(PyTypeObject *type, PyObject *args, PyObjec
 	state->len1 = PyUnicode_GET_LENGTH(arg1);
 	state->ptr1 = PyUnicode_DATA(arg1);
 	state->kind1 = PyUnicode_KIND(arg1);
+	state->transpos = transpositions;
 	  
 	return (PyObject *)state;
 }
 
 
-static PyObject * iquick_levenshtein_next(ItorState *state)
+static PyObject * ifast_comp_next(ItorState *state)
 {
 	PyObject *str2, *rv;
 	Py_ssize_t len2;
@@ -578,9 +651,7 @@ static PyObject * iquick_levenshtein_next(ItorState *state)
 	int kind2;
 	short dist = -1;
 	
-	str2 = PyIter_Next(state->itor);
-	
-	while (str2 != NULL) {
+	while ((str2 = PyIter_Next(state->itor)) != NULL) {
 	
 		if (!PyUnicode_Check(str2)) {
 			Py_DECREF(str2);
@@ -597,9 +668,11 @@ static PyObject * iquick_levenshtein_next(ItorState *state)
 		kind2 = PyUnicode_KIND(str2);
 		
 		if (state->len1 > len2)
-			dist = uni_quick_levenshtein(state->ptr1, ptr2, state->kind1, kind2, state->len1, len2);
+			dist = uni_fast_comp(state->ptr1, ptr2, state->kind1, kind2,
+				state->len1, len2, state->transpos);
 		else
-			dist = uni_quick_levenshtein(ptr2, state->ptr1, kind2, state->kind1, len2, state->len1);
+			dist = uni_fast_comp(ptr2, state->ptr1, kind2, state->kind1,
+				len2, state->len1, state->transpos);
 		
 		if (dist != -1) {
 			rv = Py_BuildValue("(hO)", dist, str2);
@@ -608,60 +681,58 @@ static PyObject * iquick_levenshtein_next(ItorState *state)
 		}
 		
 		Py_DECREF(str2);
-		str2 = PyIter_Next(state->itor);
-	
 	}
 
 	return NULL;
 }
 
 
-PyTypeObject IQuickLevenshtein_Type = {
-    PyVarObject_HEAD_INIT(&PyType_Type, 0)
-    "distance.iquick_levenshtein", /* tp_name */
-    sizeof(ItorState), /* tp_basicsize */
-    0, /* tp_itemsize */
-    (destructor)itor_dealloc, /* tp_dealloc */
-    0, /* tp_print */
-    0, /* tp_getattr */
-    0, /* tp_setattr */
-    0, /* tp_reserved */
-    0, /* tp_repr */
-    0, /* tp_as_number */
-    0, /* tp_as_sequence */
-    0, /* tp_as_mapping */
-    0, /* tp_hash */
-    0, /* tp_call */
-    0, /* tp_str */
-    0, /* tp_getattro */
-    0, /* tp_setattro */
-    0, /* tp_as_buffer */
-    Py_TPFLAGS_DEFAULT, /* tp_flags */
-    iquick_levenshtein_doc, /* tp_doc */
-    0, /* tp_traverse */
-    0, /* tp_clear */
-    0, /* tp_richcompare */
-    0, /* tp_weaklistoffset */
-    PyObject_SelfIter, /* tp_iter */
-    (iternextfunc)iquick_levenshtein_next, /* tp_iternext */
-    0, /* tp_methods */
-    0, /* tp_members */
-    0, /* tp_getset */
-    0, /* tp_base */
-    0, /* tp_dict */
-    0, /* tp_descr_get */
-    0, /* tp_descr_set */
-    0, /* tp_dictoffset */
-    0, /* tp_init */
-    PyType_GenericAlloc, /* tp_alloc */
-    iquick_levenshtein, /* tp_new */
+PyTypeObject IFastComp_Type = {
+	PyVarObject_HEAD_INIT(&PyType_Type, 0)
+	"distance.ifast_comp", /* tp_name */
+	sizeof(ItorState), /* tp_basicsize */
+	0, /* tp_itemsize */
+	(destructor)itor_dealloc, /* tp_dealloc */
+	0, /* tp_print */
+	0, /* tp_getattr */
+	0, /* tp_setattr */
+	0, /* tp_reserved */
+	0, /* tp_repr */
+	0, /* tp_as_number */
+	0, /* tp_as_sequence */
+	0, /* tp_as_mapping */
+	0, /* tp_hash */
+	0, /* tp_call */
+	0, /* tp_str */
+	0, /* tp_getattro */
+	0, /* tp_setattro */
+	0, /* tp_as_buffer */
+	Py_TPFLAGS_DEFAULT, /* tp_flags */
+	ifast_comp_doc, /* tp_doc */
+	0, /* tp_traverse */
+	0, /* tp_clear */
+	0, /* tp_richcompare */
+	0, /* tp_weaklistoffset */
+	PyObject_SelfIter, /* tp_iter */
+	(iternextfunc)ifast_comp_next, /* tp_iternext */
+	0, /* tp_methods */
+	0, /* tp_members */
+	0, /* tp_getset */
+	0, /* tp_base */
+	0, /* tp_dict */
+	0, /* tp_descr_get */
+	0, /* tp_descr_set */
+	0, /* tp_dictoffset */
+	0, /* tp_init */
+	PyType_GenericAlloc, /* tp_alloc */
+	ifast_comp, /* tp_new */
 };
 
 
 static PyMethodDef CDistanceMethods[] = {
 	{"levenshtein", (PyCFunction)levenshtein, METH_VARARGS | METH_KEYWORDS, levenshtein_doc},
 	{"hamming", (PyCFunction)hamming, METH_VARARGS | METH_KEYWORDS, hamming_doc},
-	{"quick_levenshtein", quick_levenshtein, METH_VARARGS, quick_levenshtein_doc},
+	{"fast_comp", (PyCFunction)fast_comp, METH_VARARGS | METH_KEYWORDS, fast_comp_doc},
 	{NULL, NULL, 0, NULL}
 };
 
@@ -677,11 +748,11 @@ PyMODINIT_FUNC PyInit_cdistance(void)
 	if (!module)
 		return NULL;
 
-	if (PyType_Ready(&IQuickLevenshtein_Type) != 0)
+	if (PyType_Ready(&IFastComp_Type) != 0)
 		return NULL;
 	
-	Py_INCREF((PyObject *)&IQuickLevenshtein_Type);
-	PyModule_AddObject(module, "iquick_levenshtein", (PyObject *)&IQuickLevenshtein_Type);
+	Py_INCREF((PyObject *)&IFastComp_Type);
+	PyModule_AddObject(module, "ifast_comp", (PyObject *)&IFastComp_Type);
 	
 	return module;
 }
